package com.gyee.clientside.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gyee.clientside.config.FilePathConfig;
import com.gyee.clientside.entity.Point;
import com.gyee.clientside.entity.TaosEntity;
import com.gyee.clientside.result.Result;
import com.gyee.clientside.result.ResultCode;
import com.gyee.clientside.service.FileService;
import com.gyee.clientside.service.TaosService;
import com.gyee.clientside.util.FileUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.util.StringUtil;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController()
@RequestMapping("/taos")
public class TaosController {

    @Resource
    private TaosService taosService;
    @Resource
    private FileService fileService;

    //TODO 升级用mybatis-plus

    @GetMapping("/create/database")
    private JSONObject stableCreat() {
        try {
            taosService.createDataBase();
            return Result.seccess("数据库创建成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(ResultCode.ERROR_DATA);
        }
    }

    @GetMapping("/create/supertable")
    private JSONObject stableCreat(@RequestParam(value = "aiSTablename", required = false) String aiSTablename,
                                   @RequestParam(value = "diSTablename", required = false) String diSTablename) {
        try {
            boolean b = taosService.createStables(aiSTablename, diSTablename);
            if (b) return Result.seccess("超级表创建成功！");
            return Result.error(ResultCode.ERROR_DATA);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(ResultCode.ERROR_DATA);
        }
    }

    @GetMapping("/create/table")
    private JSONObject tableCreat() {
        try {
            int tables = taosService.createTables();
            if (tables > 0) Result.seccess("点表创建成功！个数：" + tables);
            return Result.error();
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error();
        }
    }

    @GetMapping("/template")
    private void getTemplate(HttpServletResponse response) {
        FileUtils.download(FilePathConfig.getFilePath() + File.separator + "Taos更新模板.xlsx", response);
    }

    @PostMapping("/uploadexcel")
    private JSONObject uploadExcel(@RequestBody MultipartFile excelFile) {
        List<TaosEntity> entities = new ArrayList<>();
        //IOUtils.setByteArrayMaxOverride(500000000);

        try {
            FileUtils.write2File(excelFile.getInputStream());
            entities = fileService.getTaosEntitys();
            //ExcelReader reader = ExcelUtil.getReader(FilePathConfig.getTaosInfoTablePath());
            //entities = reader.read(1, 2, TaosEntity.class);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("excel读取失败!");
            return Result.error(4000, "excel读取失败!");
        }

        entities = entities.stream().map(te -> {
            te.setPoint(te.getPoint().replaceFirst("\\.", "_"));
            return te;
        }).collect(Collectors.toList());
        //redisService.creatTables(entities);

        try {
            taosService.createTablesByExcel(entities);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(4000, "建表或者修改Tag失败！");
        }
        return Result.success();
    }

    @GetMapping("/tables")
    private JSONObject getTables(@RequestParam("stablename") String stablename,
                                 @RequestParam(value = "station", required = false) String station,
                                 @RequestParam(value = "point", required = false) String point,
                                 @RequestParam(value = "description", required = false) String description,
                                 @RequestParam(value = "uniformcode", required = false) String uniformcode,
                                 @RequestParam(value = "pageNum", required = false) Integer pageNum,
                                 @RequestParam(value = "pageSize", required = false) Integer pageSize) {
        if (pageNum == null || pageSize == null) {
            pageNum = 1;
            pageSize = 100;
        }
        Page<Point> page = new Page<>(pageNum, pageSize);

        QueryWrapper wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtil.isNotBlank(station), "station", station)
                .like(StringUtil.isNotBlank(point), "tbname", point)
                .like(StringUtil.isNotBlank(description), "description", description)
                .eq(StringUtil.isNotBlank(uniformcode), "uniformcode", uniformcode);

        List<Point> points = null;
        try {
            points = taosService.selectPage(page, stablename, wrapper);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(ResultCode.ERROR_DATA);
        }

        return Result.successData(points);
    }
}
