package com.gyee.clientside.util;

import com.gyee.clientside.config.FilePathConfig;
import com.gyee.clientside.entity.TaosEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Slf4j
public class FileUtils {
    /**
     * 下载文件
     */
    public static void download(String fileName, HttpServletResponse response) {
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        try {
            // 获取文件
            File file = new File(fileName);
            // 清空缓冲区，状态码和响应头(headers)
            response.reset();
            // 设置ContentType，响应内容为二进制数据流，编码为utf-8，此处设定的编码是文件内容的编码
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            fileName = response.encodeURL(new String(file.getName().getBytes(), "iso8859-1"));
            // 以（Content-Disposition: attachment; filename="filename.jpg"）格式设定默认文件名，设定utf编码，此处的编码是文件名的编码，使能正确显示中文文件名
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);

            // 实现文件下载
            byte[] buffer = new byte[1024];
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            // 获取字节流
            OutputStream os = response.getOutputStream();
            int i = bis.read(buffer);
            while (i != -1) {
                os.write(buffer, 0, i);
                i = bis.read(buffer);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static void write2File(InputStream stream) throws IOException {
        File file = new File(FilePathConfig.getTaosInfoTablePath());
        file.delete();
        long copy = IOUtils.copy(stream, file);
        log.info("保存excel成功！" + copy);
    }

    /**
     * @describe 压缩多个文件
     */
    public static String zipFiles(List<File> srcFiles, File zipFile) {
        // 判断压缩后的文件存在不，不存在则创建
        //if (!zipFile.exists()) {
        try {
            zipFile.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
        }

        // 创建 FileOutputStream 对象
        FileOutputStream fileOutputStream = null;
        // 创建 ZipOutputStream
        ZipOutputStream zipOutputStream = null;
        // 创建 FileInputStream 对象
        FileInputStream fileInputStream = null;

        try {
            // 实例化 FileOutputStream 对象
            fileOutputStream = new FileOutputStream(zipFile);
            // 实例化 ZipOutputStream 对象
            zipOutputStream = new ZipOutputStream(fileOutputStream);
            // 创建 ZipEntry 对象
            ZipEntry zipEntry = null;
            // 遍历源文件数组
            for (int i = 0; i < srcFiles.size(); i++) {
                // 将源文件数组中的当前文件读入 FileInputStream 流中
                fileInputStream = new FileInputStream(srcFiles.get(i));
                // 实例化 ZipEntry 对象，源文件数组中的当前文件
                zipEntry = new ZipEntry(srcFiles.get(i).getName());
                zipOutputStream.putNextEntry(zipEntry);
                // 该变量记录每次真正读的字节个数
                int len;
                // 定义每次读取的字节数组
                byte[] buffer = new byte[1024];
                while ((len = fileInputStream.read(buffer)) > 0) {
                    zipOutputStream.write(buffer, 0, len);
                }
            }
            zipOutputStream.closeEntry();
            zipOutputStream.close();
            fileInputStream.close();
            fileOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        //}
        return zipFile.getAbsolutePath();
    }

    public static List<TaosEntity> excel2Entity(XSSFWorkbook xssfWorkbook) {
        log.info("读取excel!");
        List<TaosEntity> entities = new ArrayList<>();
        SXSSFWorkbook workbook = new SXSSFWorkbook(xssfWorkbook, 9999);
        SXSSFSheet sheet = workbook.getSheetAt(0);
        //获得当前sheet的结束行
        int lastRowNum = sheet.getLastRowNum();
        String scv1, scv2, scv3, scv4, scv5, scv6, scv8, scv9;
        float scv7;
        TaosEntity te;
        //循环从第2行开始
        for (int rowNum = 2; rowNum <= lastRowNum; rowNum++) {
            //获得当前行
            Row row = sheet.getRow(rowNum);
            scv1 = row.getCell(0).getStringCellValue();
            scv2 = row.getCell(1).getStringCellValue();
            scv3 = row.getCell(2).getStringCellValue();
            scv4 = row.getCell(3).getStringCellValue();
            scv5 = row.getCell(4).getStringCellValue();
            scv6 = row.getCell(5).getStringCellValue();
            scv7 = (float) row.getCell(6).getNumericCellValue();
            scv8 = row.getCell(7).getStringCellValue();
            scv9 = row.getCell(8).getStringCellValue();
            te = new TaosEntity();
            te.setPoint(scv1);
            te.setDescription(scv2 != null ? scv2 : "");
            te.setStation(scv3);
            te.setCategory(scv4 != null ? scv4 : "");
            te.setFacility(scv5 != null ? scv5 : "");
            te.setUniformcode(scv6);
            te.setRate(scv7);
            te.setRemark(scv8 == null ? "" : scv8);
            te.setDatatype(scv9);
            entities.add(te);
        }
        log.info("excel读取完毕！");
        return entities;
    }
}
