package com.gyee.redisread.service;

import cn.hutool.core.collection.ListUtil;
import com.gyee.feign.api.Redis2TaosApi;
import com.gyee.redisread.config.RedisConfig;
import com.gyee.redisread.task.TaskRedisRead;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.DefaultApplicationArguments;
import org.springframework.core.annotation.Order;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Slf4j
@Order(1)
@Service
public class ReadRedisService implements ApplicationRunner {

    @Resource
    private RedisService redisService;
    @Resource
    private ThreadPoolTaskExecutor taskExecutor;
    @Resource
    private RedisConfig redisConfig;
    @Resource
    private Redis2TaosApi redis2TaosApi;

    List<String> dikeys = new ArrayList<>();
    List<String> aikeys = new ArrayList<>();

    @Value("${active:false}")
    private boolean active;

    @Override
    public void run(ApplicationArguments args) {

        getKeys();

        List<List<String>> boolStrll = ListUtil.split(dikeys, 9216);
        List<List<String>> douStrll = ListUtil.split(aikeys, 9216);

        while (active) {
            try {
                for (List<String> strings : boolStrll) {
                    taskExecutor.execute(new TaskRedisRead(strings, redisConfig, "bool", redis2TaosApi));
                }
                for (List<String> strings : douStrll) {
                    taskExecutor.execute(new TaskRedisRead(strings, redisConfig, "double", redis2TaosApi));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            log.info("\n#");
        }
    }

    public void getKeys() {
        //long l1 = System.currentTimeMillis();
        Set<String> keys = redisService.getKeys();
        if (keys.size() == 0) return;
        //long l2 = System.currentTimeMillis();
        log.info("读取rediskeys成功！！！有" + keys.size() + "个点。");
        //按类型分点
        dikeys = new ArrayList<>();
        aikeys = new ArrayList<>();
        keys.forEach(k -> {
            if (k.contains("DI") || k.contains("di")) {
                dikeys.add(k);
            } else if (k.contains("AI") || k.contains("ai")) {
                aikeys.add(k);
            }
        });
    }

    public boolean stop() {
        this.active = false;
        return true;
    }

    public boolean start() {
        this.active = true;
        run(new DefaultApplicationArguments());
        return true;
    }

    public boolean isActive() {
        return active;
    }

    public List<String> getDikeys() {
        return dikeys;
    }

    public List<String> getAikeys() {
        return aikeys;
    }
}
