package com.gyee.redisread.task;

import com.gyee.feign.api.Redis2TaosApi;
import com.gyee.redisread.config.RedisConfig;
import com.gyee.redisread.entity.PointData;
import com.gyee.redisread.util.ByteUtil;
import com.gyee.redisread.util.ChangedSave;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.Pipeline;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class TaskRedisRead implements Runnable {
    private final List<String> keys;
    private RedisConfig redisConfig;
    private String dataType;
    private Redis2TaosApi redis2TaosApi;

    public static final Logger log = LoggerFactory.getLogger(TaskRedisRead.class);

    public TaskRedisRead(List<String> keys, RedisConfig redisConfig, String dataType, Redis2TaosApi redis2TaosApi) {
        this.keys = keys;
        this.redisConfig = redisConfig;
        this.dataType = dataType;
        this.redis2TaosApi = redis2TaosApi;
    }

    private byte[] binaryStringToByteArray(String binaryString) {
        byte[] byteArray = new byte[binaryString.length() / 8];
        for (int i = 0; i < byteArray.length; i++) {
            String byteString = binaryString.substring(i * 8, i * 8 + 8);
            byteArray[i] = (byte) Integer.parseInt(byteString, 2);
        }
        return byteArray;
    }

    private void getPointData(List<Object> objects, StringBuilder sb) {
        boolean b = "bool".equals(dataType);  // 提前计算，只判断一次
        for (int i = 0; i < objects.size(); i++) {
            String response = (String) objects.get(i);
            String key = keys.get(i);
            byte[] bytes = binaryStringToByteArray(response);
            double val = ByteUtil.getDouble(bytes, 0);
            long ts = ByteUtil.getLong2(bytes, 8);
            Long ts0 = ChangedSave.map.get(key);

            if (ts0 == null || ts != ts0) {
                ChangedSave.map.put(key, ts);
                if (ts0 != null) {  // 只有在 ts0 != null 时才追加字符串
                    if (b) {
                        sb.append(key).append(" VALUES (").append(ts).append(",").append(val != 0).append(") ");
                    } else {
                        sb.append(key).append(" VALUES (").append(ts).append(",").append(val).append(") ");
                    }
                }
            }
        }
    }

    @Override
    public void run() {

        List<Object> objects;

        String pointid = null;

        try (Jedis jedis = redisConfig.getJedis(); Pipeline pipelined = jedis.pipelined();) {

            for (String key : keys) {
                //pipelined.hscan(key, "0");
                //pipelined.hmget(key, "pointid", "value", "timestamp");
                pipelined.mget(key);
            }
            //redis批量获取接收为Object
            objects = pipelined.syncAndReturnAll();

            StringBuilder sb = new StringBuilder();
            sb.append("INSERT INTO ");

            //String value;
            //String datetime;
            //double val;
            //long ts;
            //boolean bl;
            //List<String> response;
            getPointData(objects, sb);
            String sb2str = sb.toString();
            if ("INSERT INTO ".equals(sb2str)) return;
            for (String taosUrl : redisConfig.getTaosUrls().split(",")) {
                redis2TaosApi.putSql(URI.create(taosUrl), sb2str);
            }

            /*if ("bool".equals(dataType)) {
                for (int i = 0; i < objects.size(); i++) {
                    try {
                        response = (List<String>) objects.get(i);
                    }catch (Exception e){
                        log.error(objects.get(i).toString());
                        continue;
                    }

                    value = response.get(1);
                    if (value == null || "".equals(value) || value.contains("-nan")) continue;
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if (ChangedSave.map.containsKey(pointid)) {
                        if (!Objects.equals(ChangedSave.map.get(pointid), datetime)) {
                            ChangedSave.map.put(pointid, datetime);
                            ts = datetime.length()>10?Long.valueOf(datetime): Long.valueOf(datetime)* 1000;
                            try{
                                bl = Double.parseDouble(value) == 1;
                            }catch (Exception e){
                                log.warn(value);
                                continue;
                            }
                            //j++;
                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(bl).append(") ");
                        }
                    } else {
                        ChangedSave.map.put(pointid, datetime);
                    }
                    //ts = Long.valueOf(datetime) * 1000;
                    //bl = Double.valueOf(value) == 1;
                    //sb.append(pointid).append(" VALUES (").append(ts).append(",").append(bl).append(") ");
                }
                sb2str = sb.toString();
                if ("INSERT INTO ".equals(sb2str)) return;
                for (String taosUrl : redisConfig.getTaosUrls().split(",")) {
                    redis2TaosApi.putSql(URI.create(taosUrl), sb2str);
                }
            } else {
                for (int i = 0; i < objects.size(); i++) {
                    try {
                        response = (List<String>) objects.get(i);
                    }catch (Exception e){
                        log.error(objects.get(i).toString());
                        continue;
                    }

                    value = response.get(1);
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if (ChangedSave.map.containsKey(pointid)) {
                        if (!Objects.equals(ChangedSave.map.get(pointid), datetime)) {
                            ChangedSave.map.put(pointid, datetime);
                            ts = datetime.length()>10?Long.valueOf(datetime): Long.valueOf(datetime)* 1000;
                            try{
                                val = Double.parseDouble(value);
                                if (Double.isNaN(val) || Double.isInfinite(val)) continue;
                            }catch (Exception e){
                                log.warn(value);
                                continue;
                            }
                            //j++;
                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(val).append(") ");
                        }
                    } else {
                        ChangedSave.map.put(pointid, datetime);
                    }
                    //ts = Long.valueOf(datetime) * 1000;
                    //val = Double.valueOf(value);
                    //sb.append(pointid).append(" VALUES (").append(ts).append(",").append(val).append(") ");
                }
                sb2str = sb.toString();
                if ("INSERT INTO ".equals(sb2str)) return;
                for (String taosUrl : redisConfig.getTaosUrls().split(",")) {
                    redis2TaosApi.putSql(URI.create(taosUrl), sb2str);
                }
            }*/
        } catch (Exception e) {
            e.printStackTrace();
            log.error(pointid + "------写请求超时或读取redis出错！");
        }
        System.out.print("*");
    }
}
